const fs = require("fs");
const util = require("util");
const readdir = util.promisify(fs.readdir);
const Rss_parser = require("rss-parser");
var rss_parser = new Rss_parser();
var xml_parser = require("xml2js").parseString;
const args = require("yargs")
.usage("Usage : Milestone2/run.js -only test -file only on this file -v")
.alias("f", "file")
.alias("o", "only")
.alias("v", "verbose")
.describe("file", "If you want to run the test on only one file" +
"(as opposed to all files in the directory), " +
"add this option and specify file location")
.describe("only", "If you only want to run one test, " +
"add this option and specify the test file name (rss/sitemap)")
.describe("verbose", "Print results to console, " +
"if not specified results will only be copied to result file")
.nargs("file", 1)
.nargs("only", 1)
.nargs("verbose", 0)
.argv;

var to_run_rss = [];
var rss_stream;
var to_run_sitemap = [];
var sitemap_stream;
var only_script = args.only == null
? ""
: args.only;
var only_file = args.file == null
? ""
: args.file;

function run(script, files, stream, callback){
    var childProcess = require("child_process");
    if (files.length === 0){
        callback();
        return;
    }

    var invoked = false,
    to_run = files.shift();
    var new_process = childProcess.fork(script, ["-f", to_run]);

    new_process.on("message", function (mes){
        mes = mes.toString();
        if (args.v) console.log(to_run + " : " + mes);
        fs.readFile(to_run, (err, data) => {
            if (to_run.includes("rss") || to_run.includes("atom")){
                rss_parser.parseString(data, (err, result) => {
                    if(err) stream.write(to_run + "," + "bad formatting" + "," + mes + "\n");
                    else stream.write(to_run +","+ result.items.length +","+ mes + "\n");
                });
            } else {
                xml_parser(data, (err, result) => {
                    if(err){
                        var data_lines = data.toString().split("\n");
                        if (data_lines[0].includes("https://") || data_lines[0].includes("http://"))
                            stream.write(to_run + "," + data_lines.length + "," + mes + "\n")
                        else stream.write(to_run + "," + "bad formatting" + "," + mes + "\n");
                    } else {
                        if (result.urlset) stream.write(to_run +","+ result.urlset.url.length.toString() +","+ mes + "\n");
                        else stream.write(to_run + "," + "bad formatting" + "," + mes + "\n");
                    }
                });
            }
        });
        run(script, files, stream, callback);
    });

    new_process.on("error", function (err) {
        if (invoked) return;
        invoked = true;
        callback(err);
    });

    // new_process.on("exit", function () {
    //     callback();
    // });
}

function run_sitemap(){
    run("./unit_tests/test_sitemap.js", to_run_sitemap, sitemap_stream, function (){return;});
}

async function prep_rss(){
    fs.truncateSync("./results/rss.csv");
    rss_stream = fs.createWriteStream("./results/rss.csv", {flags:"a"});
    rss_stream.write("file name,items,items in index,title found,content complete\n");
    var rss_files = await readdir("./files/RSS");
    rss_files.forEach(file => {
        if (file.includes(only_file)) to_run_rss.push("./files/RSS/" + file);
    });
}

async function prep_sitemap(){
    fs.truncateSync("./results/sitemap.csv");
    sitemap_stream = fs.createWriteStream("./results/sitemap.csv", {flags:"a"});
    sitemap_stream.write("file name,items,items in index,title found,content complete\n");
    var sitemap_files = await readdir("./files/SITEMAP");
    sitemap_files.forEach(file => {
        if (file.includes(only_file)) to_run_sitemap.push("./files/SITEMAP/" + file);
    });
}

(async () => {
    if (only_script.includes("rss")){
        await prep_rss();
        run("./unit_tests/test_rss.js", to_run_rss, rss_stream, function (){return;});
    } else if (only_script.includes("sitemap")){
        await prep_sitemap();
        run_sitemap();
    } else {
        await prep_rss();
        await prep_sitemap();
        run("./unit_tests/test_rss.js", to_run_rss, rss_stream, run_sitemap);
    }

})();
